@@ -595,6 +595,12 @@ clone_image_menu_size (GtkImage *image, GtkSettings *settings)
       gtk_image_get_stock (image, &stock_id, NULL);
       return gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_MENU);
     }
+  else if (storage_type == GTK_IMAGE_ICON_NAME)
+    {
+      const gchar *icon_name;
+      gtk_image_get_icon_name (image, &icon_name, NULL);
+      return gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
+    }
   else if (storage_type == GTK_IMAGE_ICON_SET)
     {
       GtkIconSet *icon_set;
@@ -906,6 +912,8 @@ gtk_tool_button_set_label (GtkToolButton *button,
 			   const gchar   *label)
 {
   gchar *old_label;
+  gchar *elided_label;
+  AtkObject *accessible;
   
   g_return_if_fail (GTK_IS_TOOL_BUTTON (button));
 
@@ -914,6 +922,14 @@ gtk_tool_button_set_label (GtkToolButton *button,
   button->priv->label_text = g_strdup (label);
   button->priv->contents_invalid = TRUE;     
 
+  if (label)
+    {
+      elided_label = _gtk_toolbar_elide_underscores (label);
+      accessible = gtk_widget_get_accessible (GTK_WIDGET (button->priv->button));
+      atk_object_set_name (accessible, elided_label);
+      g_free (elided_label);
+    }
+
   g_free (old_label);
  
   g_object_notify (G_OBJECT (button), "label");
